@@ -27,9 +27,9 @@ LOG = logging.getLogger(__name__)
 
 
 # key is a deprecated version and value is an alternative version.
-DEPRECATED_VERSIONS = {"1": "2"}
+DEPRECATED_VERSIONS = {"2": "3"}
 DEPRECATED_VERSION = "2.0"
-MAX_VERSION = "3.58"
+MAX_VERSION = "3.59"
 MIN_VERSION = "3.0"
 
 _SUBSTITUTIONS = {}
@@ -160,6 +160,9 @@ class APIVersion(object):
             return "%s.%s" % (self.ver_major, "latest")
         return "%s.%s" % (self.ver_major, self.ver_minor)
 
+    def get_major_version(self):
+        return "%s" % self.ver_major
+
 
 class VersionedMethod(object):
 
@@ -275,19 +278,32 @@ def discover_version(client, requested_version):
     server_start_version, server_end_version = _get_server_version_range(
         client)
 
-    valid_version = requested_version
     if not server_start_version and not server_end_version:
         msg = ("Server does not support microversions. Changing server "
                "version to %(min_version)s.")
         LOG.debug(msg, {"min_version": DEPRECATED_VERSION})
-        valid_version = APIVersion(DEPRECATED_VERSION)
-    else:
-        valid_version = _validate_requested_version(
-            requested_version,
-            server_start_version,
-            server_end_version)
+        return APIVersion(DEPRECATED_VERSION)
+
+    _validate_server_version(server_start_version, server_end_version)
+
+    # get the highest version the server can handle relative to the
+    # requested version
+    valid_version = _validate_requested_version(
+        requested_version,
+        server_start_version,
+        server_end_version)
+
+    # see if we need to downgrade for the client
+    client_max = APIVersion(MAX_VERSION)
+    if client_max < valid_version:
+        msg = _("Requested version %(requested_version)s is "
+                "not supported. Downgrading requested version "
+                "to %(actual_version)s.")
+        LOG.debug(msg, {
+            "requested_version": requested_version,
+            "actual_version": client_max})
+        valid_version = client_max
 
-        _validate_server_version(server_start_version, server_end_version)
     return valid_version
 
 
