@@ -43,7 +43,7 @@ class ShellTest(utils.TestCase):
     FAKE_ENV = {
         'OS_USERNAME': 'username',
         'OS_PASSWORD': 'password',
-        'OS_TENANT_NAME': 'tenant_name',
+        'OS_PROJECT_NAME': 'tenant_name',
         'OS_AUTH_URL': 'http://no.where/v2.0',
     }
 
@@ -201,6 +201,12 @@ class ShellTest(utils.TestCase):
         _shell = shell.OpenStackCinderShell()
         _shell.main(['list'])
 
+    def test_duplicate_filters(self):
+        _shell = shell.OpenStackCinderShell()
+        self.assertRaises(exceptions.CommandError,
+                          _shell.main,
+                          ['list', '--name', 'abc', '--filters', 'name=xyz'])
+
     @unittest.skip("Skip cuz I broke it")
     def test_cinder_service_name(self):
         # Failing with 'No mock address' means we are not
@@ -227,7 +233,7 @@ class ShellTest(utils.TestCase):
             self.FAKE_ENV['OS_AUTH_URL'],
             password=mock_getpass.return_value,
             tenant_id='',
-            tenant_name=self.FAKE_ENV['OS_TENANT_NAME'],
+            tenant_name=self.FAKE_ENV['OS_PROJECT_NAME'],
             username=self.FAKE_ENV['OS_USERNAME'])
 
     @requests_mock.Mocker()
