@@ -51,6 +51,8 @@ class ShellTest(utils.TestCase):
             self.useFixture(fixtures.EnvironmentVariable(var,
                                                          self.FAKE_ENV[var]))
 
+        self.mock_completion()
+
         self.shell = shell.OpenStackCinderShell()
 
         self.requests = self.useFixture(requests_mock_fixture.Fixture())
@@ -103,7 +105,7 @@ class ShellTest(utils.TestCase):
                                'metadata': {'key1': '"--test1"'},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False}}
+                               }}
         self.assert_called_anytime('POST', '/volumes', expected)
 
     def test_metadata_args_limiter_display_name(self):
@@ -119,7 +121,7 @@ class ShellTest(utils.TestCase):
                                'metadata': {'key1': '"--t1"'},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False}}
+                               }}
         self.assert_called_anytime('POST', '/volumes', expected)
 
     def test_delimit_metadata_args(self):
@@ -135,7 +137,7 @@ class ShellTest(utils.TestCase):
                                             'key2': '"test2"'},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False}}
+                               }}
         self.assert_called_anytime('POST', '/volumes', expected)
 
     def test_delimit_metadata_args_display_name(self):
@@ -151,7 +153,7 @@ class ShellTest(utils.TestCase):
                                'metadata': {'key1': '"t1"'},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False}}
+                               }}
         self.assert_called_anytime('POST', '/volumes', expected)
 
     def test_list_filter_status(self):
@@ -217,37 +219,11 @@ class ShellTest(utils.TestCase):
         mock_print.assert_called_once_with(mock.ANY, key_list,
             exclude_unavailable=True, sortby_index=0)
 
-    def test_list_sort_valid(self):
-        self.run_command('list --sort_key=id --sort_dir=asc')
-        self.assert_called('GET', '/volumes/detail?sort_dir=asc&sort_key=id')
-
-    def test_list_sort_key_name(self):
-        # Client 'name' key is mapped to 'display_name'
-        self.run_command('list --sort_key=name')
-        self.assert_called('GET', '/volumes/detail?sort_key=display_name')
-
     def test_list_sort_name(self):
         # Client 'name' key is mapped to 'display_name'
         self.run_command('list --sort=name')
         self.assert_called('GET', '/volumes/detail?sort=display_name')
 
-    def test_list_sort_key_invalid(self):
-        self.assertRaises(ValueError,
-                          self.run_command,
-                          'list --sort_key=foo --sort_dir=asc')
-
-    def test_list_sort_dir_invalid(self):
-        self.assertRaises(ValueError,
-                          self.run_command,
-                          'list --sort_key=id --sort_dir=foo')
-
-    def test_list_mix_sort_args(self):
-        cmds = ['list --sort name:desc --sort_key=status',
-                'list --sort name:desc --sort_dir=asc',
-                'list --sort name:desc --sort_key=status --sort_dir=asc']
-        for cmd in cmds:
-            self.assertRaises(exceptions.CommandError, self.run_command, cmd)
-
     def test_list_sort_single_key_only(self):
         self.run_command('list --sort=id')
         self.assert_called('GET', '/volumes/detail?sort=id')
@@ -275,10 +251,7 @@ class ShellTest(utils.TestCase):
 
     def test_list_reorder_with_sort(self):
         # sortby_index is None if there is sort information
-        for cmd in ['list --sort_key=name',
-                    'list --sort_dir=asc',
-                    'list --sort_key=name --sort_dir=asc',
-                    'list --sort=name',
+        for cmd in ['list --sort=name',
                     'list --sort=name:asc']:
             with mock.patch('cinderclient.utils.print_list') as mock_print:
                 self.run_command(cmd)
@@ -1129,16 +1102,6 @@ class ShellTest(utils.TestCase):
         self.assert_called('POST', '/volumes/1234/action',
                            body={'os-unmanage': None})
 
-    def test_replication_promote(self):
-        self.run_command('replication-promote 1234')
-        self.assert_called('POST', '/volumes/1234/action',
-                           body={'os-promote-replica': None})
-
-    def test_replication_reenable(self):
-        self.run_command('replication-reenable 1234')
-        self.assert_called('POST', '/volumes/1234/action',
-                           body={'os-reenable-replica': None})
-
     def test_create_snapshot_from_volume_with_metadata(self):
         """
         Tests create snapshot with --metadata parameter.
